# frozen_string_literal: true

# can be hardcoded here but for copy the example around that is easier
require Dir["#{File.dirname(__FILE__)}/*_jars.rb"].first
require Dir["#{File.dirname(__FILE__)}/*jar"].first
