# frozen_string_literal: true

# this is a generated file, to avoid over-writing it just delete this comment
require 'jar_dependencies'

require_jar('org.bouncycastle', 'bcpkix-jdk15on', '1.49')
require_jar('org.bouncycastle', 'bcprov-jdk15on', '1.49')
