# frozen_string_literal: true

# this require will load ALL dependent jars or the jars from Jars.lock if
# the lock file exeists
require 'example_jars'

# this is the jar compiled from src/main/java/**
require 'example.jar'
